/*
 *
 * StackFHex: Mainly a x86 shellcode development assistance tool.
 * written by dr.raid, drraid at gmail dot com
 *
 * Purpose: this tool automates the tedious task of little endian reversing, 
 * pushing, and padding data to the stack in x86 assembly.
 *
 * How: this tool reads from stdin, until EOF (or out of memory)
 * and prints out little-endian arranged 4-byte push statements
 * of all the data read, padding the data to a 4byte boundary with "0x41" at the 
 * *BEGINNING* at the string on the stack (last push statement made).
 *
 * 
 * More: Read the README for further data.
 * Feel free and distribute/use as
 * permitted in the GNU/GPL -- ;]
 *
 * questions/comments/insults/etc?
 * drraid gmail com
 * 
 * 
 * -dR
 *
*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/select.h>
#include <error.h>

#define STDIN 0
#define MAX_BUF 128


/* Function prototypes */
void intro(void);
void print_help(char *s);
struct line_node *get_input(void);
void show_output(struct line_node *first);
struct line_node *addstring(struct line_node *node, char *s, int cnt);
struct line_node *rebuildlist(struct line_node *node, int offset);
struct line_node *concatstr(struct line_node *dst, struct line_node *src);
struct line_node *newnode(void);
void clear_nodes(struct line_node *first);
void hx_push(struct line_node *node);
void die(int line);

/* simple double linked list node */
struct line_node
{
	/* string */
	char buf[MAX_BUF];	
	/* stored len */
	int len;

	/* */
	int offset;
	
	/* links */
	struct line_node *next;
	struct line_node *prev;
};

/* code */

int main(int argc, char **argv)
{ 	 
	/* pointer to linkedlist of input */
	struct line_node *list;
	
	/* fd_set for polling stdin */
	fd_set r_set;

	/* timeout = 0 */
	struct timeval tv;
	
	/* setup the fd set */	
	FD_ZERO(&r_set);
	FD_SET(STDIN, &r_set);

	/* set no timeout */
	tv.tv_sec = 0;
	tv.tv_usec = 0;

	/* show intro */
	intro();

	/* is there input ready?  */
	if (0 == select(1, &r_set, NULL, NULL, &tv))
	{

		/* did they want help? */
		if ((argv[1]) && (0 == strncmp(argv[1], "-h", 2)))
		{
			print_help(argv[0]);
			die(__LINE__);
		}

		/* prompt then get_data */
		printf("\ninput> ");
		fflush(stdout);
	}

	/* get input returns first node in linked list */
	list = get_input();

	/* print output */
	show_output(list);	

	/* free memory */
	clear_nodes(list);

	fprintf(stderr, "    execution: complete\n\n");

	/* die */
	die(__LINE__);

	/* this is never reached.. */
	return 0;
}

/* ascii art: every g00d hax0r needs it
 * onlyy printed on stderr: so stdout is saved for
 * clean redirection of ASM ;]
*/
void intro(void)
{
	fprintf(stderr, 
		"        __               __     _______ __                \n");
	fprintf(stderr, 
		".-----.|  |_.---.-.----.|  |--.|    ___|  |--.-----.--.--.\n");
	fprintf(stderr, 
		"|__ --||   _|  _  |  __||    < |    ___|     |  -__|_   _|\n");
	fprintf(stderr, 
		"|_____||____|___._|____||__|__||___|   |__|__|_____|__.__|\n");
	fprintf(stderr, 
		"    Stack-F-hex: x86 stack formatted hex output utility   \n");
	fprintf(stderr, 
		"    drraid gmail com\n\n");
	return;
}

/* help output */
void print_help(char *s)
{
	printf("\nStack-Formatted Hex utility\n");
	printf("----------------------------\n");
	printf("purpose: format strings into x86 asm push statements\n");
	printf("primarily used for shellcode/small app developement\n");
	printf("----------------------------\n");
	printf("usage:\n");
	printf("\tStandard user-prompt:\t%s\n", s);
	printf("\tPiping from a file:\tcat filename | %s\n", s);
	printf("\n");
	return;
}

/* get_input: this function takes no arguments.
 * this reads input from stdin in sizes of MAX_BUF, until EOF
 * data is stored in a linked list, and a pointer to the first
 * node is returned */
struct line_node *get_input(void)
{
	/* input buffer */
	char buf[MAX_BUF];

	/* input counter */
	int cnt = 0;
	int lcnt= 0;

	/* loop counter */
	int loop = 0;

	/* pointers: first and current node */
	struct line_node *first=NULL, *cur=NULL;

	/* clean memory */
	memset(buf, 0x0, sizeof(buf));	

	/* get memory */
	cur = first = newnode();
	
	/* while data was read.. */
	while ( 0 < (cnt = read(STDIN, buf, sizeof(buf))))
	{
		/* add it to our linked list */
		cur = addstring(cur, buf, cnt);
		/* store last count */
		lcnt = cnt;
		/* clean or memory */
		memset(buf, 0x0, sizeof(buf));	
	}

	/* prepend padding: if it doesnt land on a 4byte boundary
	 * we have to rebuild the list so add-on padding is before
	 * the string, not after
	*/ 
	if (lcnt % 4)
		first = rebuildlist(first, (4 - (lcnt % 4)));

	/* return pointer to first node */
	return first;
}

/* show_output: transcend the list backwards
 * passing each node's buffer to hx_push();
*/
void show_output(struct line_node *first)
{
	struct line_node *p1, *p2;
	int cnt = 0;

	/* set pointer to first of list */
	p1 = first;
	while (p1->next) p1=p1->next;

	/* while there is data */
	while (p1)
	{
		cnt+=p1->len - p1->offset;

		/* print it in 4-byte push statements */
		hx_push(p1);

		/* goto the next node */
		p1 = p1->prev;
	}

	
	/* now go backwards through the list printing
	 * each node out */

	printf("; %i bytes of data\n", cnt);
	if (cnt % 4)
		printf("; is located at esp+%i\n", 4 - (cnt % 4));
	
	return;
}


/* addstring: 
 * alloc a new node if necessary - copy in the string
 * and set the node's len variable to cnt */
struct line_node *addstring(struct line_node *node, char *s, int cnt)
{	
	struct line_node *n1=node, *n2=NULL;

	/* if its the first node, we have memory, but its 
	 * been memset to NULLs, so check that first */
	if (n1->buf[0] != 0x0)
	{
		/* this node has data,
		 * and must not be the first.. */

		/* get a new one */
		n1->next=newnode();

		/* set up link structure */
		n2 = n1->next;
		n2->prev = n1;
		n1 = n2;
	}
	
	/* copy in string */
	strncpy(n1->buf, s, cnt);

	/* set the len */
	n1->len = cnt;

	/* return the new node */
	return n1;
}

/* newnode: alloc a new node */
struct line_node *newnode(void)
{
	struct line_node *p1=NULL;
	
	/* allocate memory: there should really be some
	 * error checking here.. im kinda lazy */
	if ( p1 = malloc(sizeof(struct line_node)))
	{
		/* clean our memory */
		memset((char *)p1, 0x0, sizeof(struct line_node));
	}
	return p1;
}

/* clear_nodes: this is to free the entire
 * list, starting at the first node */
void clear_nodes(struct line_node *first)
{
	/* p1 = first, p2 = tmp */
	struct line_node *p1, *p2;

	p1 = first;

	while (p2 = p1->next)
	{
		free(p1);
		p1 = p2;
	}
	
	free(p1);

	return;
}

void die(int line)
{
	exit(line);
}

/* hx_push: print out the node->buf data
 * as 4-byte asm push statements */
void hx_push(struct line_node *node)
{
	int i, x, len;	
	unsigned char *p1, *p2;
	

	/* get the remainder, if there is one */
	if (i = (node->len % 4))
	{
		/* fix the lengths */

		i = 4 - i;
		len = node->len + i;

		/* pad the remaining space with x86 NOPs 
		 * up until the boundary marker */
		while (i--)	
			node->buf[(node->len + i)]=0x90;

	}
	else
	{
		/* theres no remainder, len = cnt */
		len = node->len;
	}

	/* set beginning and end pointer */
	p1 = node->buf;
	p2 = node->buf+len - 4;

	/* move through the buffer, print 4 bytes at a time */
	for (p2 ; p2 >= p1; p2-=4)
		printf("push 0x%02x%02x%02x%02x\n", 
				p2[3], p2[2], p2[1], p2[0]);
			
	return;
}


/* 
 * rebuild list: 
 * re-arranges the data stored in each buffer of each linked node pointed to by 'node'
 * then frees the entire list pointed to by node, returning the first node in the new list; 
*/
struct line_node *rebuildlist(struct line_node *node, int offset)
{
	struct line_node *p1 = node, *p2 = NULL, *p3 = NULL, *keep = NULL;
	int loop = 0, already_there=0, available = 0;
	char buffer[MAX_BUF];
	char *cur = NULL;
 
	/* KEEP the first node to pass to 'clear nodes' at the end */
	keep = p1 = node;
	/* keep the first pointer to the new list in p3 */
	p3 = p2 = newnode();

	/* 
	 * set our 'offset' to be 0x41:
	 * this would normally be 0x90 for a traditional x86-style NOP Slide,
	 * but 0x41 is ascii printable ('A'), and is the op for inc ecx.. which
	 * adds one to the value in ecx.. being as any reasonable shellcode probably
	 * clean the contents of required registeres, using 0x41 works as a better suited
	 * "POP" slide: 'pointless op' to lead up to shellcode, as much more likely to look
	 * sketchy (sincne we've all seen \x90\x90\x90... in our logfiles) ;] 
	*/ 

	memset(p2->buf, 0x41, offset);

	/* set the len/offset of the p2 buffer.. 
	 * this is used for the re-arrangement logic later on */
	p2->len = offset;	
	p2->offset = offset;

	/* while the node != NULL */
	while (p1)
	{
		/* concat the data to our current list, adding nodes as necissary */
		p2 = concatstr(p2, p1);
		/* get the next node to add */
		p1 = p1->next;
	}

	/* remove the old list */
	clear_nodes(keep);

	/* return first pointer to our new list */
	return p3;
}

/* concat str: takes buffer data in src node and adds it to list pointed to by dst,
 * adding a new node if required */
struct line_node *concatstr(struct line_node *dst, struct line_node *src)
{
	/* local node pointers */
	struct line_node *dest = NULL, *temp = NULL;		

	/* pointer for buffer offsets */
	char *p1 = NULL;

	/* space available in buffer */
	int space_avail = 0;

	/* remaining to copy into new node */
	int to_copy = 0;

	/* set our local pointer */
	dest = dst;

	/* calc space available */
	space_avail = (MAX_BUF - dst->len);

	/* offset by data in dest buf */
	p1 = &dest->buf[dest->len];
	
	/* is data left in the node? */
	if (src->len > space_avail)
	{
		to_copy = src->len - space_avail;
		memcpy(p1, src->buf, space_avail);
		dest->len += space_avail;
		dest->next = newnode();
		temp = dest->next;
		temp->prev = dest;
		p1 = &src->buf[space_avail];
		memcpy(temp->buf, p1, to_copy);
		temp->len += to_copy;
		return temp;
	}
	

	memcpy(p1, src->buf, src->len);
	dest->len += src->len;

	if (space_avail == src->len)
	{
		dest->next = newnode();
		temp = dest->next;
		temp->prev = dest;
		return temp;
	}
	return dest;
	
}

